# -*- coding: utf-8 -*-

from functools import wraps


class TestCaseMediator(object):
    """
    Mediator class between noseapp.suite.Suite, TestLoader, TestSuite,
    TestCase. Provides between TestCase->TestLoader->TestSuite.
    """

    def __init__(self, require):
        self._test_cases = []
        self._require = require
        self._handlers = []

    @property
    def require(self):
        return self._require

    @property
    def test_cases(self):
        return self._test_cases

    @property
    def handlers(self):
        return self._handlers

    def create_map(self):
        """
        {
            'class name': {
                'cls': 'link to class object',
                'tests': {
                    'method name': 'link to class method',
                },
            },
        }

        :return: dict
        """
        mp = {}

        for case in self._test_cases:
            mp[case.__name__] = {
                'cls': case,
                'tests': dict(
                    (atr, getattr(case, atr))
                    for atr in dir(case)
                    if atr.startswith('test')
                    or
                    atr.startswith('runTest'),
                ),
            }

        return mp

    def create_suite(self, program_data):
        """
        Create suite instance
        """
        suite = program_data.suite_class(
            config=program_data.config,
            handlers=self._handlers,
        )

        for case in self._test_cases:
            suite.addTests(
                program_data.test_loader.loadTestsFromTestCase(
                    case.setup_extensions(self._require),
                ),
            )

        return suite

    def add_test_case(self, test_case):
        """
        Remember test case class
        """
        self._test_cases.append(test_case)

    def add_handler(self, f):
        """
        Add run test handler
        """
        self._handlers.append(f)
        return wraps(f)



# -*- coding: utf8 -*-

from unittest import TestCase


class TestClearExtensions(TestCase):
    """
    Set extension, check get extension, call clear, check raise
    """

    def runTest(self):
        from noseapp.core import extensions

        extensions.set('test', dict(test='test'))
        self.assertEqual(extensions.get('test').get('test'), 'test')

        extensions.clear()
        self.assertRaises(extensions.ExtensionNotFound, extensions.get, 'test')


class TestClassFactory(TestCase):
    """
    Test ClassFactory usage
    """

    def runTest(self):
        from noseapp.runner.base import BaseTestRunner
        from noseapp.suite.bases.simple import BaseSuite
        from noseapp.suite.bases.threading_suite import ThreadSuite
        from noseapp.runner.threading_runner import ThreadingTestRunner
        from noseapp.runner.multiprocessing_runner import MultiprocessingTestRunner

        from noseapp.core.factory import ClassFactory

        class MockOptions(object):
            def __init__(self, **kw):
                self.thread_pool = kw.pop('thread_pool', 0)
                self.app_processes = kw.pop('app_processes', 0)
                self.gevent_greenlets = kw.pop('gevent_greenlets', 0)
                self.gevent_pool = kw.pop('gevent_pool', 0)

        factory = ClassFactory(MockOptions())

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, BaseTestRunner)

        factory = ClassFactory(MockOptions(app_processes=2))

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)

        factory = ClassFactory(MockOptions(thread_pool=2))

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, ThreadingTestRunner)

        factory = ClassFactory(MockOptions(app_processes=2, thread_pool=2))

        self.assertEqual(factory.suite_class, ThreadSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)

        factory = ClassFactory(MockOptions(app_processes=2, gevent_greenlets=2))

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)



# -*- coding: utf-8 -*-

from noseapp.case.base import TestCase
from noseapp.case.screenplay import step
from noseapp.case.screenplay import ScreenPlayCase
from noseapp.datastructures import ModifyDict as _BaseCaseSettings


class TestCaseSettings(_BaseCaseSettings):
    """
    Installation settings for test case from application

    Usage:
        settings = TestCaseSettings(
            debug=True,
            project_url='http://...',
        )
        settings.install(app)

        suite = Suite(__name__, require=['settings'])
    """

    name = 'settings'

    def install(self, app):
        app.shared_data(self.name, self)


__all__ = (
    step,
    TestCase,
    ScreenPlayCase,
    TestCaseSettings,
)




import os

from noseapp.plugins.base import AppPlugin


class AppConfigurePlugin(AppPlugin):

    name = 'app-configure'

    def options(self, parser, env=os.environ):
        super(AppConfigurePlugin, self).options(parser, env=env)

        parser.add_option(
            '--app-processes',
            dest='app_processes',
            default=0,
            type=int,
            help='Num of processes for running. one process == one suite.'
        )
        parser.add_option(
            '--gevent',
            dest='gevent_pool',
            default=0,
            type=int,
            help='Num of greenlets for running. One greenlet == one suite.'
        )
        parser.add_option(
            '--greenlets',
            dest='gevent_greenlets',
            default=0,
            type=int,
            help='Num of greenlets for running. One greenlet == one test case.'
        )
        parser.add_option(
            '--threads',
            dest='thread_pool',
            default=0,
            type=int,
            help='Num of threads for running. If app processes is set, one thread == one test case, '
                 'else one thread == one suite.'
        )
        parser.add_option(
            '--ls',
            dest='ls',
            action='store_true',
            default=False,
            help='Show suites tree.'
        )
        parser.add_option(
            '--doc',
            dest='doc',
            action='store_true',
            default=False,
            help='Show docstring of test cases.'
        )
        parser.add_option(
            '-t', '--test',
            dest='run_test',
            default='',
            type=str,
            help='Test case or suite name for running.'
        )

        try:  # if app instance does not pushed to plugin,
            # AttributeError will be passed.
            self.app.parser = parser
            self.app.add_options()
        except AttributeError:
            pass



# -*- coding: utf8 -*-

from noseapp.core.program import TestProgram
from noseapp.core.collector import CollectSuite
from noseapp.core.loader import load_suites_from_path


class ExtensionInstaller(object):
    """
    Container for extension.
    Lazy creating instance.
    """

    def __init__(self, cls, args, kwargs):
        self.name = cls.name

        self.cls = cls
        self.args = args
        self.kwargs = kwargs

    def __call__(self):
        return self.cls(*self.args, **self.kwargs)


__all__ = (
    TestProgram,
    CollectSuite,
    ExtensionInstaller,
    load_suites_from_path,
)
